% Simulates three ice core datasets and calculates the spectral estimate using
% GLOSA, described in Svedberg, D. Elvander, F. Jakobsson, A.
% "Determining Joint Periodicities in Multi-time Data", 2021

% Saves the simulated data in iddata object 'dataseti' for examination. 

% Note: Check README file for additional information on included datasets
% and required add-ons.
clear; close all;
%% Simulation settings
numSets = 3; % Number of datasets to be used 
noise = true; % Add noise
missamp = true; % Sampling uncertainty?
plotData = true; % Plot original data?
plotIt = true; % Plot estimates
zoomIter = 4; % Number of zooming steps. 
NBands = 16; % Number of initial frequency bands.
plotZoom = false; % Plot zooming steps from GLOSA.
lambda = 5; % Weights in GLOSA cost function
zeta = 10*ones(numSets,1); % Weights in GLOSA cost function

%% Settings for periodogram-based methods
% Number of grid points
NF = 2^14;
% Frequency grid
ff = 0.1*(0:NF-1)'/NF;

%% Simulate datasets 
% Name of dataset (can be changed, must then define what other sampling
% pattern the data set should be based on). 
datasets = {'vostok.mat', 'taylorDome.mat', 'domefuji.mat'};
fprintf(['Simulating three data sets based on "'  datasets{1} '",  "'  datasets{2} '", and "' datasets{3} '" respectively. \n' ...
    'Sampling Instants drawn from a Poisson mixture distribution calculated in samplinginstants.m\n\n']);

% Frequencies from orbital theory of climate (may be changed to arbitrary
% values)
freq = [1/100 1/41 1/23 1/19]';
numFreq = length(freq);
% Choose signal-to-noise ratio 
SNR = 5;

% Choose base amplitudes
orig_amplitude = [1 .8 .6 .6]';
origAms = sum(orig_amplitude.^2/2);
ampDev = 0.2; % Amplitude std. dev. - percentage of orig_amplitude 
% Noise variance based on signal power and SNR
noisevar = 10^(-SNR/10)*origAms;

% Pre-allocate variables for drawn amplitudes, phases, and sampling
% instants
amplitudes = zeros(numFreq, numSets);
phases = zeros(numFreq, numSets);

% Keep track of current figure number
numPlots = 0;

% Store simulated data in cell with iddata objects. 
allData = cell(numSets,1);
% Store Lomb-Scargle periodograms in matrix.
LSper = zeros(NF, numSets);
for i = 1:numSets
    % Different signal amplitudes, same overall power. 0.1 may be changed
    % to a larger deviation if wanted. 
    amp_mismatch = sqrt(ampDev*orig_amplitude).*randn(numFreq, 1);
    thisAmp = orig_amplitude+amp_mismatch;
    amplitudes(:,i) = sqrt(origAms./sum(thisAmp.^2/2)).*thisAmp;
    % Draw phases 
    phases(:,i) = pi*rand(numFreq, 1);
    fprintf(['Dataset ' num2str(i) '\t amplitudes =' num2str(amplitudes(:,i)') '\n \t\t     phases =' num2str(phases(:,i)') '\n\n'])
    % The assumed sampling times, may differ from nominal sampling times if
    % missamp = true
    [assumedT,trueT] = samplinginstants(datasets{i}, missamp);
    % Number of samples
    n = length(assumedT);
    % Simulate signal w. white noise
    y = cos(2*pi*freq'.*assumedT+phases(:,i)')*amplitudes(:,i)+sqrt(noisevar)*randn(n,1);
    % Zero-mean
    y = y-mean(y);
    allData{i} = iddata(y, [], 'SamplingInstants', assumedT);
    % Must use sorted data for Lomb-Scargle periodogram implementation,
    % with non-negative times. 
    [LST, I] = sort(assumedT-min(assumedT));
    LSper(:,i) = plomb(y(I)./length(y), LST, 'power', ff);
    if plotData
        figure(numPlots+1)
        hold on
        plot(assumedT, y, 'LineWidth',1);
    end
end 
meanPower = mean(amplitudes.^2/2,2);
numPlots = numPlots +1;
hold off;
grid on; 
xlabel('t [kyBP]', 'FontSize',14);
ylabel('y(t)', 'FontSize',14);
l1 = legend('Data 1', 'Data 2', 'Data 3');
l1.FontSize = 12;

%% Estimate frequencies and spectra using the three methods. 
% First estimate using GLOSA
if plotZoom
    strAppend = 'true';
else
    strAppend = 'false';
end
fprintf(['Estimating using GLOSA with ' num2str(zoomIter) ' zooming iterations and ' ...
    num2str(NBands) ' initial bands.\nPlot zoom iterations = ' strAppend '\nTime: '])
tic;
[GLOSASpect, GLOSAFreq] = GLOSA(allData,0.1,NBands,zoomIter,numFreq,plotZoom,numPlots+1, lambda, zeta);
toc;
if plotZoom
    numPlots = numPlots + 1;
end
% Estimate using mean of Lomb-Scargle periodograms
fprintf('\nEstimate using mean Lomb-Scargle periodogram \nTime: ')
tic;
meanSpect = mean(LSper, 2);
meanFreq = ff(findpeaks(meanSpect, numFreq));
toc;

fprintf('\nEstimate using stacking method \nTime: ')
% Estimate using stacking method
tic;
[stackSpect, stackFreq] = createStack(allData, ff, numFreq);
toc;
fprintf('\n')
%% Show results 
% NOTE: THIS IS ONLY FOR VISUAL COMPARISON. The goal of this method is not
% to estimate amplitudes, it is only used for visual comparison, and is
% therefore scaled to be of similar magnitude. Remember that the mean
% periodogram is normalized by the number of data point to ensure that each
% data set contributes equally much to the estimate. 
figure(numPlots+1)
plot(GLOSAFreq,GLOSASpect./max(GLOSASpect), 'sq');
hold on
plot(ff, meanSpect./max(meanSpect), '--','LineWidth',1.5); 
plot(ff, stackSpect./max(stackSpect), '-', 'LineWidth',1.5);
plot(freq, meanPower./max(meanPower), 'o');
for i = 1:numFreq
   line(freq(i)*ones(2,1),[0;meanPower(i)./max(meanPower)]);
   line(GLOSAFreq(i)*ones(2,1), [0;GLOSASpect(i)], 'LineWidth', 1.5); 
end
hold off
grid on
legend('GLOSA', 'Mean LS periodogram','Stacking Method', 'True frequencies', 'FontSize', 14)
xlabel('Frequency [1/kyr]','FontSize',14);
ylabel('Normalized power','FontSize',14);
%% Locally defined functions
% Implementation of stack method
function [spect, freq] = createStack(allData,ff, Na)
% function [spect, freq] = createStack(allData,ff, Na)
% Number of data sets
numSets = length(allData);
% Normalize all data and stack
stackY = [];
stackT = [];
for i = 1:numSets
    allData{i}.y = (allData{i}.y-mean(allData{i}.y))./std(allData{i}.y);
    % Collect all signal values in a single variable
    stackY = [stackY; allData{i}.y];
    % Collect all sampling instants in a single variable
    stackT = [stackT; allData{i}.SamplingInstants];
end
% Sort all values by sampling instant. 
[stackT,I] = sort(stackT);
stackY = stackY(I);

stackT = stackT(numSets:end);
stackY = stackY(numSets:end);

numSamp = length(stackT);

% Using method explained in: 
% Imbrie, J. & Hays, J. & Martinson, 
% Douglas & McIntyre, A. & Mix, Alan & Morley, J. & Pisias, 
% Nicklas & Prell, W. & Shackleton, N.. (1984). 
% "The orbital theory of Pleistocene climate: Support from a revised 
% chronology of the marine delta 18O record." 

% Resampling to equidistant sampling
Fs = numSamp/stackT(end);
[stackY, stackT] = resample(stackY, stackT, Fs);
% Smoothing with 9 point Gaussian filter
w = gausswin(9);
w = w./sum(w);
stackY = filter(w,1,stackY);

% Calculate periodogram. 
spect = plomb(stackY,stackT,'power',ff);
posspect = spect(ff>=0);
ffpos = ff(ff>=0);
% Calculate frequency estimates
freq = ffpos(sort(findpeaks(posspect, Na)));
end
