% Non-homgeneous poisson process will model sampling times.
% However, to simulate th behaviour of data where sampling 
% frequency is fairly constant during recent years, use 
% MLE of intensity function for data earlier in time, but
% constant intensity for recent times. 

% Detect change point using normal distribution of ML estimate. 

function [misT, sampT] = samplinginstants(dataset, missamp)
% Which dataset do you wish to base sampling on? Other cores could of
% course be used! Then a .mat file that can be loaded by samplinginstants
% must be included. 
if isequal(dataset, 'vostok.mat')
    load('vostok.mat', 'vostok');
    realT = vostok.SamplingInstants;
elseif isequal(dataset, 'gisp.mat')
    load('gisp.mat', 'gispT');
    realT = gispT;
elseif isequal(dataset, 'domefuji.mat')
    load('domefuji.mat', 'domefuji');
    realT = domefuji.SamplingInstants;
elseif isequal(dataset, 'taylorDome.mat')
    load('taylorDome.mat', 'taylorDome');
    realT = taylorDome.SamplingInstants;
else
    error('dataset not recognized');
end
% Number of samples
n = length(realT);
% Pre-allocate variable
sampT = zeros(n,1);

% Initiate change point
T_c = NaN;
indx = 2;
% Initiate confidence interval. 
CI = [-Inf, Inf];
% Run until change point is found
while isnan(T_c)&&indx<n
    % Estimate constant intensity 
    lambda_MLE = (indx-1)./(realT(indx)-realT(1));
    % Is it outside the CI? Then it is the change point
    if (CI(1)>lambda_MLE)||(CI(2)<lambda_MLE)
        T_c = realT(indx-1);
    end
    % Otherwise, calculate new CI. 
    dev = norminv(0.95)*sqrt(lambda_MLE/(indx-1));
    CI = [lambda_MLE-dev, lambda_MLE+dev];
%     figure(1)
%     hold on
%     scatter(indx,lambda_MLE, 'ro');
%     scatter(indx, CI(1), 'k.');
%     scatter(indx, CI(2), 'k.')
%     hold off
    indx = indx+1;
end
if indx < n
% This is where the change point occured.
switchindx = indx-2;
switchtime = realT(switchindx);
% Conditional on number of events in interval, homogeneous Poisson process
% is distributed as the order statistics of a uniform distribution. 
sampT(1:switchindx) = sort(switchtime*rand(switchindx,1));

% Part of distribution fitted to NHPP
NHPPT = realT(switchindx+1:end);
nNHPP = length(NHPPT);
switchtime = NHPPT(1);
NHPPT = NHPPT - switchtime;
% Approximate intensity function using ML.
MLapprox = @(params)MLmidApprox(params, NHPPT);
params = fminsearch(MLapprox, [1, -1]);

u = rand(nNHPP,1);
% Draw sampling times for the remaining interval from the NHPP 
sampTNHPP = log(u*(exp(params(2)*realT(end))-1)+1)/params(2);
sampT(switchindx+1:end) = sort(sampTNHPP) + switchtime;
elseif indx == n
    % Conditional on number of events in interval, homogeneous Poisson process
    % is distributed as the order statistics of a uniform distribution.
    sampT = sort(realT(end)*rand(n,1));
else 
    error('Error when drawing sampling instants, is the Poisson mixture model reasonable for this case?');
end
% Pre-allocate variables to store times with missampling. 
misT = zeros(n,1);
misT(1) = sampT(1);
% Draw missampling
if missamp
    % Probability that two samples change place
    threshold = 0.99;
    for i = 2:n
        if i<n
            fct = @(x)abs(erf((sampT(i+1)-sampT(i))/sqrt(2)/x)-erf((sampT(i-1)-sampT(i))/sqrt(2)/x)-2*threshold);
            sigma = fminbnd(fct, 0, 1000);
        else
            fct = @(x)abs( 1-erf((sampT(i-1)-sampT(i))/sqrt(2)/x)-2*threshold);
            sigma = fminbnd(fct, 0, 1000);
        end
        misT(i) = normrnd(sampT(i), sigma);
    end
else
    misT = sampT;
end
end


function l = MLmidApprox(alpha,T)
lambda = exp(alpha(1)+alpha(2)*T(2:end));
term1 = -sum(lambda.*diff(T));
term2 = sum(log(lambda)+log(diff(T)));
l = -(term1+term2);
end